import { List, ActionPanel, Action } from "@raycast/api";
import { useEffect, useState } from "react";
import fs from "fs";
import path, { resolve } from "path";
import { homedir } from "os";

const BASE_PATH = resolve(homedir(), "dev/grizzlebit.com/content/");
const BLOG_SUBDIRS = ["posts", "notes", "links"];

type Post = {
  file: string;
  content: string;
};

type SearchResult = {
  file: string;
  snippet: string;
};

function getAllMarkdownFiles(dir: string): string[] {
  let results: string[] = [];
  const list = fs.readdirSync(dir);
  list.forEach((file) => {
    const filePath = path.join(dir, file);
    const stat = fs.statSync(filePath);
    if (stat && stat.isDirectory()) {
      results = results.concat(getAllMarkdownFiles(filePath));
    } else if (file.endsWith(".md")) {
      results.push(filePath);
    }
  });
  return results;
}

function getAllPosts(root: string, subdirs: string[]): Post[] {
  const files = subdirs.flatMap((subdir) =>
    getAllMarkdownFiles(path.join(root, subdir))
  );
  return files.map((filePath) => {
    const content = fs.readFileSync(filePath, "utf8");
    return { file: filePath, content };
  });
}

function highlightSnippet(line: string, query: string): string {
  const regex = new RegExp(`(${query})`, "gi");
  return line.replace(regex, "**$1**");
}

function searchPosts(posts: Post[], query: string): SearchResult[] {
  const lowerQuery = query.toLowerCase();
  return posts
    .map(({ file, content }) => {
      const matchedLine = content
        .split("\n")
        .find((line) => line.toLowerCase().includes(lowerQuery));

      if (matchedLine) {
        const snippet = highlightSnippet(matchedLine, query);
        return { file, snippet };
      }

      return null;
    })
    .filter((result): result is SearchResult => result !== null);
}

export default function Command() {
  const [query, setQuery] = useState<string>("");
  const [results, setResults] = useState<SearchResult[]>([]);

  useEffect(() => {
    if (query.trim() === "") {
      setResults([]);
      return;
    }
    try {
      const posts = getAllPosts(BASE_PATH, BLOG_SUBDIRS);
      const matches = searchPosts(posts, query);
      setResults(matches);
    } catch (err) {
      console.error("Error reading blog posts:", err);
    }
  }, [query]);

  return (
    <List onSearchTextChange={setQuery} throttle isShowingDetail>
      {results.map(({ file, snippet }) => {
        const filename = path.basename(file);
        const relativePath = path.relative(BASE_PATH, file).replace(/\\/g, "/");

        // Convert to URL relative from site root based upon Hugo URL config
        const relativeUrl = `/${relativePath
          .replace(/\.md$/, "")
          .replace(/\/\d\d\d\d-/, "/")}`;

        // Grab the title from the front matter
        const fileContent = fs.readFileSync(file, "utf8");
        const titleMatch = fileContent.match(/^title:\s*(.*)$/m);
        const title = titleMatch
          ? titleMatch[1].replace(/^['"]|['"]$/g, "")
          : filename;
        const markdownLink = `[${title}](${relativeUrl})`;

        return (
          <List.Item
            key={file}
            title={snippet.replace(/\*\*/g, "")}
            detail={
              <List.Item.Detail
                markdown={`**${relativeUrl}**\n\n---\n\n${snippet}`}
              />
            }
            actions={
              <ActionPanel>
                <Action.Open
                  title="Open in VS Code"
                  target={file}
                  application="/Applications/Visual Studio Code.app"
                />
                <Action.CopyToClipboard
                  title="Copy Markdown Link"
                  content={markdownLink}
                />
              </ActionPanel>
            }
          />
        );
      })}
    </List>
  );
}
